package asm

type Mnemonic uint8

const (
	NONE Mnemonic = iota
	ADD
	CALL
	COMMENT
	COMPARE
	DIV
	JE
	JNE
	JG
	JGE
	JL
	JLE
	JUMP
	MUL
	LABEL
	LOAD
	MOVE
	POP
	PUSH
	RETURN
	STORE
	SUB
	SYSCALL
)

// String returns a human readable version.
func (m Mnemonic) String() string {
	switch m {
	case ADD:
		return "add"
	case CALL:
		return "call"
	case COMMENT:
		return "comment"
	case COMPARE:
		return "compare"
	case DIV:
		return "div"
	case JUMP:
		return "jump"
	case JE:
		return "jump if =="
	case JNE:
		return "jump if !="
	case JL:
		return "jump if <"
	case JG:
		return "jump if >"
	case JLE:
		return "jump if <="
	case JGE:
		return "jump if >="
	case LABEL:
		return "label"
	case LOAD:
		return "load"
	case MOVE:
		return "move"
	case MUL:
		return "mul"
	case POP:
		return "pop"
	case PUSH:
		return "push"
	case RETURN:
		return "return"
	case SUB:
		return "sub"
	case STORE:
		return "store"
	case SYSCALL:
		return "syscall"
	default:
		return ""
	}
}
