package core

import (
	"strconv"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/token"
)

// CompileAssign compiles an assign statement.
func (f *Function) CompileAssign(node *ast.Assign) error {
	operator := node.Expression.Token
	left := node.Expression.Children[0]
	right := node.Expression.Children[1]

	if left.IsLeaf() {
		name := left.Token.Text(f.File.Bytes)
		variable := f.VariableByName(name)

		if variable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, left.Token.Position)
		}

		defer f.useVariable(variable)
		return f.Execute(operator, variable.Register, right)
	}

	if left.Token.Kind == token.Array {
		name := left.Children[0].Token.Text(f.File.Bytes)
		variable := f.VariableByName(name)

		if variable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, left.Children[0].Token.Position)
		}

		defer f.useVariable(variable)

		index := left.Children[1]
		offset, err := strconv.Atoi(index.Token.Text(f.File.Bytes))

		if err != nil {
			return err
		}

		num, err := strconv.Atoi(right.Token.Text(f.File.Bytes))

		if err != nil {
			return err
		}

		f.MemoryNumber(asm.STORE, asm.Memory{Base: variable.Register, Offset: byte(offset), Length: 1}, num)
		return nil
	}

	return errors.New(errors.NotImplemented, f.File, left.Token.Position)
}
