package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/cpu"
)

func (f *Function) AddLabel(label string) {
	f.Assembler.Label(asm.LABEL, label)
	f.postInstruction()
}

func (f *Function) Call(label string) {
	f.Assembler.Call(label)
	f.CurrentScope().Use(f.cpu.Output[0])
	f.postInstruction()
}

func (f *Function) Jump(mnemonic asm.Mnemonic, label string) {
	f.Assembler.Label(mnemonic, label)
	f.postInstruction()
}

func (f *Function) MemoryNumber(mnemonic asm.Mnemonic, a asm.Memory, b int) {
	f.Assembler.MemoryNumber(mnemonic, a, b)
	f.postInstruction()
}

func (f *Function) Register(mnemonic asm.Mnemonic, a cpu.Register) {
	f.Assembler.Register(mnemonic, a)

	if mnemonic == asm.POP {
		f.CurrentScope().Use(a)
	}

	f.postInstruction()
}

func (f *Function) RegisterNumber(mnemonic asm.Mnemonic, a cpu.Register, b int) {
	if f.CurrentScope().IsUsed(a) && isDestructive(mnemonic) {
		f.SaveRegister(a)
	}

	f.Assembler.RegisterNumber(mnemonic, a, b)

	if mnemonic == asm.MOVE {
		f.CurrentScope().Use(a)
	}

	f.postInstruction()
}

func (f *Function) RegisterLabel(mnemonic asm.Mnemonic, register cpu.Register, label string) {
	if f.CurrentScope().IsUsed(register) && isDestructive(mnemonic) {
		f.SaveRegister(register)
	}

	f.Assembler.RegisterLabel(mnemonic, register, label)

	if mnemonic == asm.MOVE {
		f.CurrentScope().Use(register)
	}

	f.postInstruction()
}

func (f *Function) RegisterRegister(mnemonic asm.Mnemonic, a cpu.Register, b cpu.Register) {
	if mnemonic == asm.MOVE && a == b {
		return
	}

	if f.CurrentScope().IsUsed(a) && isDestructive(mnemonic) {
		f.SaveRegister(a)
	}

	f.Assembler.RegisterRegister(mnemonic, a, b)

	if mnemonic == asm.MOVE {
		f.CurrentScope().Use(a)
	}

	f.postInstruction()
}

func (f *Function) Return() {
	f.Assembler.Return()
	f.postInstruction()
}

func (f *Function) Syscall() {
	f.Assembler.Syscall()
	f.CurrentScope().Use(f.cpu.Output[0])
	f.postInstruction()
}

func isDestructive(mnemonic asm.Mnemonic) bool {
	switch mnemonic {
	case asm.MOVE, asm.ADD, asm.SUB, asm.MUL, asm.DIV:
		return true
	default:
		return false
	}
}
