package core

import (
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/config"
	"git.akyoto.dev/cli/q/src/build/cpu"
)

// SaveRegister attempts to move a variable occupying this register to another register.
func (f *Function) SaveRegister(register cpu.Register) {
	if !f.CurrentScope().IsUsed(register) {
		return
	}

	for _, general := range f.cpu.General {
		if register == general {
			return
		}
	}

	variable := f.VariableInRegister(register)

	if variable == nil || variable.Alive == 0 {
		return
	}

	newRegister := f.CurrentScope().MustFindFree(f.cpu.General)
	f.CurrentScope().Reserve(newRegister)

	if config.Comments {
		f.Comment("save %s to %s", register, newRegister)
	}

	f.RegisterRegister(asm.MOVE, newRegister, register)
	variable.Register = newRegister
}
