package scope

import (
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/token"
)

type Stack struct {
	Scopes []*Scope
}

// CurrentScope returns the current scope.
func (stack *Stack) CurrentScope() *Scope {
	return stack.Scopes[len(stack.Scopes)-1]
}

// PopScope removes the scope at the top of the stack.
func (stack *Stack) PopScope() {
	stack.Scopes = stack.Scopes[:len(stack.Scopes)-1]
}

// PushScope pushes a new scope to the top of the stack.
func (stack *Stack) PushScope(body ast.AST, buffer []byte) *Scope {
	s := &Scope{}

	if len(stack.Scopes) > 0 {
		lastScope := stack.Scopes[len(stack.Scopes)-1]
		s.State = lastScope.State
		s.Variables = make(map[string]*Variable, len(lastScope.Variables))
		s.InLoop = lastScope.InLoop

		for k, v := range lastScope.Variables {
			count := ast.Count(body, buffer, token.Identifier, v.Name)

			if count == 0 {
				continue
			}

			s.Variables[k] = &Variable{
				Name:     v.Name,
				Register: v.Register,
				Alive:    count,
			}
		}
	} else {
		s.Variables = map[string]*Variable{}
	}

	stack.Scopes = append(stack.Scopes, s)
	return s
}
